#version 330

uniform sampler2D color_texture;
uniform sampler2D normals_texture;
uniform sampler2D depth_texture;

uniform mat3 G[9] = mat3[](
	1.0/(2.0*sqrt(2.0)) * mat3(1.0, sqrt(2.0), 1.0, 0.0, 0.0, 0.0, -1.0, -sqrt(2.0), -1.0),
	1.0/(2.0*sqrt(2.0)) * mat3(1.0, 0.0, -1.0, sqrt(2.0), 0.0, -sqrt(2.0), 1.0, 0.0, -1.0),
	1.0/(2.0*sqrt(2.0)) * mat3(0.0, -1.0, sqrt(2.0), 1.0, 0.0, -1.0, -sqrt(2.0), 1.0, 0.0),
	1.0/(2.0*sqrt(2.0)) * mat3(sqrt(2.0), -1.0, 0.0, -1.0, 0.0, 1.0, 0.0, 1.0, -sqrt(2.0)),
	1.0/2.0 * mat3(0.0, 1.0, 0.0, -1.0, 0.0, -1.0, 0.0, 1.0, 0.0),
	1.0/2.0 * mat3(-1.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, -1.0),
	1.0/6.0 * mat3(1.0, -2.0, 1.0, -2.0, 4.0, -2.0, 1.0, -2.0, 1.0),
	1.0/6.0 * mat3(-2.0, 1.0, -2.0, 1.0, 4.0, 1.0, -2.0, 1.0, -2.0),
	1.0/3.0 * mat3(1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0)
);

out vec4 fragment_color;

void main(void)
{
	mat3 I;
	float cnv[9];
	float sum = 0;

	for (int i = 0; i < 3; i++)
		for (int j = 0; j < 3; j++) {
			float d_ref = texelFetch(depth_texture, ivec2(gl_FragCoord), 0).r;
			float d = texelFetch(depth_texture, ivec2(gl_FragCoord) + ivec2(i - 1, j - 1), 0).r;
			float diff = d - d_ref;
			sum += abs(diff);
			I[i][j] = diff;
		}
	
	float c = 0;

	// test if we are looking at a flat surface
	float avg = sum / 9;
	if (avg < 0.00001) {
		fragment_color = vec4(c, c, c, 1);
		return;
	}

	/* calculate the convolution values for all the masks */
	for (int i = 0; i < 9; i++) {
		float dp3;
		
		dp3 = dot(G[i][0], I[0]) + dot(G[i][1], I[1]) + dot(G[i][2], I[2]);
		cnv[i] = dp3 * dp3;
	}

	float M, S;

	M = (cnv[0] + cnv[1]) + (cnv[2] + cnv[3]);
	S = (cnv[4] + cnv[5]) + (cnv[6] + cnv[7]) + (cnv[8] + M);
	float depth_derivation = abs(sqrt(M / S) - 1);

	// both vectors are in camera space!
	//vec3 inverted_view = vec3(0, 0, 1);
	//vec3 normal = texelFetch(normals_texture, ivec2(gl_FragCoord), 0).rgb * 2 - 1;
	// dot(v1, v2) = 1 ... if the two vectors are parallel

	if (depth_derivation > 0.0001)// && dot(normal, inverted_view) < 1)
		c = 1;

	fragment_color = vec4(c, c, c, 1);
}
